/**
 * Copyright (c) 2022, WSO2 LLC. (https://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 LLC. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content.
 */

import React, { MutableRefObject, ReactElement, UIEventHandler, useEffect, useState } from "react";
import { Accordion, Icon } from "semantic-ui-react";
import LoaderPlaceholder from "./loader-placeholder";
import { InterfaceDiagnosticLogEntry, InterfaceLogsFilter } from "../models";
import { formatTimestampToDateTime, getDateFromTimestamp, getTimeFromTimestamp } from "../utils/datetime-utils";

interface InfiniteScrollContainerPropsInterface {
    handleScroll: UIEventHandler<HTMLDivElement>,
    scrollRef: MutableRefObject<HTMLDivElement>,
    logs: any[],
    loading: boolean,
    rowHeight: number,
    logCount: number,
    isPreviousEmpty?: boolean,
    isNextEmpty?: boolean,
    setSearchQuery?: (query: InterfaceLogsFilter) => void
}

/**
 * Infinite scrolling component for diagnostic logs
 * @param props - InfiniteScrollContainer props
 * @returns React functional component
 */
const InfiniteScrollContainer = (props: InfiniteScrollContainerPropsInterface): ReactElement => {
    const {
        handleScroll,
        scrollRef,
        logs,
        loading,
        rowHeight,
        logCount,
        setSearchQuery
    } = props;

    const [ activeIndex, setActiveIndex ] = useState(-1);

    useEffect(() => {
        /**
         * scroll the container to the first log
         */
        if (logs.length > 0 && logs.length <= logCount) {
            scrollRef.current.scrollTop = rowHeight * (logCount - (logCount-1));
        }
        /**
         * Collapse detailed log view
         */
        setActiveIndex(-1);
    }, [ logs ]);

    const handleClick = (e, titleProps) => {
        const { index } = titleProps;
        const newIndex = activeIndex === index ? -1 : index;

        setActiveIndex(newIndex);
    };

    const handleExpandedView = (logObject: InterfaceDiagnosticLogEntry) => {
        const propertyElements: Array<ReactElement> = [];

        for (const property in logObject) {
            if (property === "id" || property === "componentId") continue;
            if (logObject[property]) {
                if (typeof logObject[property] === "string") {
                    if (property === "recordedAt") {
                        propertyElements.push(
                            <p>
                                <span className="log-property">{ property }:</span>
                                &nbsp;&nbsp;&nbsp;&nbsp;{ formatTimestampToDateTime(logObject[property]) }
                            </p>
                        );
                    } else if (property === "resultStatus") {
                        propertyElements.push(
                            <p>
                                <span className="log-property">{ property }:</span>
                                &nbsp;&nbsp;&nbsp;&nbsp;{ logObject[property] }
                            </p>
                        );
                    } else {
                        propertyElements.push(
                            <p>
                                <span className="log-property">{ property }:</span>
                                &nbsp;&nbsp;&nbsp;&nbsp;{ logObject[property] }
                                &nbsp;&nbsp;
                                <Icon.Group
                                    className="icon-button"
                                    onClick={ () => {
                                        setSearchQuery({ property: property, value: logObject[property] });
                                    } }
                                >
                                    <Icon name="filter" color="green" />
                                    <Icon corner name="add" color="green" />
                                </Icon.Group>
                            </p>
                        );
                    }
                } else if (typeof logObject[property] === "object") {
                    for (const childProp in logObject[property])
                        propertyElements.push(
                            <p>
                                <span className="log-property">{ childProp }:</span>
                                &nbsp;&nbsp;&nbsp;&nbsp;{ logObject[property][childProp] }
                                &nbsp;&nbsp;
                                <Icon.Group
                                    className="icon-button"
                                    onClick={ () => {
                                        setSearchQuery({ property: childProp, value: logObject[property][childProp] });
                                    } }
                                >
                                    <Icon name="filter" color="green" />
                                    <Icon corner name="add" color="green" />
                                </Icon.Group>
                            </p>
                        );
                }
            }
        }

        return propertyElements;
    };

    return (
        <div className="main-container">
            <div
                className="ui fluid container scroll-container"
                onScroll={ handleScroll }
                ref={ scrollRef }
            >
                <div className="edge-container"></div>
                { loading && (
                    <LoaderPlaceholder />
                ) }
                <Accordion fluid>
                    { logs.map((log, key) => (
                        <div key={ key }>
                            <Accordion.Title
                                active={ activeIndex === key }
                                index={ key }
                                onClick={ handleClick }
                                style={ { padding: "0px" } }
                            >
                                <div className="log-row">
                                    <div className="log-time-container">
                                        { getDateFromTimestamp(log.recordedAt) }
                                        <span className="log-time">
                                            &nbsp;|&nbsp;
                                            { getTimeFromTimestamp(log.recordedAt) }
                                        </span>
                                    </div>
                                    <div className="log-description-container">
                                        <div className="log-message-container">
                                            { log.resultStatus === "SUCCESS" &&
                                                <Icon name="check circle" color="green" />
                                            }
                                            { log.resultStatus === "FAILED" &&
                                                <Icon name="close" color="red" />
                                            }
                                            { log.resultMessage }
                                        </div>
                                        { log.actionId && (
                                            <div className="log-actionid-container">
                                                { log.actionId }
                                            </div>
                                        ) }
                                    </div>
                                </div>
                            </Accordion.Title>
                            <Accordion.Content active={ activeIndex === key }>
                                { handleExpandedView(log).map((el, key) => (
                                    <div className="log-expanded-view" key={ key }>
                                        { el }
                                    </div>
                                )) }
                            </Accordion.Content>
                        </div>
                    )) }
                </Accordion>
                { loading && (
                    <LoaderPlaceholder />
                ) }
                <div className="edge-container"></div>
            </div>
        </div>
    );
};

export default InfiniteScrollContainer;
