/**
 * Copyright (c) 2022, WSO2 LLC. (https://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 LLC. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content.
 */

import { IdentifiableComponentInterface } from "@wso2is/core/models";
import { SearchUtils } from "@wso2is/core/utils";
import { AdvancedSearch, EmptyPlaceholder, LinkButton, ListLayout, PageLayout } from "@wso2is/react-components";
import { getEmptyPlaceholderIllustrations } from "apps/console/src/features/core";
import React, {
    FunctionComponent,
    MutableRefObject,
    ReactElement,
    useEffect,
    useRef,
    useState
} from "react";
import { useTranslation } from "react-i18next";
import {  Dropdown, Icon, Label } from "semantic-ui-react";
import AdvancedFilterPopup from "../components/advanced-filter-popup";
import InfiniteScrollContainer from "../components/infinite-scroll-container";
import { LogsConstants } from "../constants";
import useFetch from "../hooks/use-fetch";
import { InterfaceDiagnosticLogsRequest, InterfaceLogsFilter } from "../models/diagnostic-log-models";
import { getDateTimeWithOffset } from "../utils";

/**
 * Proptypes for the logs page component.
 */
type LogsPageInterface = IdentifiableComponentInterface;

/**
 * Number of logs fetched for a request.
 */
const LOG_FETCH_COUNT: number = LogsConstants.LOG_FETCH_COUNT;

/**
 * Height of a single log row in collapsed state.
 */
const LOG_ROW_HEIGHT: number = LogsConstants.LOG_ROW_HEIGHT;

const TIMERANGE_DROPDOWN_OPTIONS = LogsConstants.TIMERANGE_DROPDOWN_OPTIONS;

/**
 * Logs monitoring page.
 *
 * @param props - {@link LogsPageInterface} Props injected to the component.
 *
 * @returns Logs Page {@link React.ReactElement}
 */
const LogsPage: FunctionComponent<LogsPageInterface> = (
    props: LogsPageInterface
): ReactElement => {
    const { ["data-componentid"]: componentId } = props;

    const [ isPreviousEmpty, setIsPreviousEmpty ] = useState<boolean>(false);
    const [ isNextEmpty, setIsNextEmpty ] = useState<boolean>(false);
    const [ searchQuery, setSearchQuery ] = useState<string>("");
    const [ filterQuery, setFilterQuery ] = useState<string>("");
    const [ triggerClearQuery, setTriggerClearQuery ] = useState<boolean>(false);
    const [ isAddFilterFormSubmitted, setIsAddFilterFormSubmitted ] = useState<boolean>(false);
    const [ endTime, setEndTime ] = useState<string>("");
    const [ startTime, setStartTime ] = useState<string>("");
    const [ timeZone ] = useState<string>("GMT+0000 UTC");
    const [ timeRange, setTimeRange ] = useState<number>(0.5);
    const [ filterMap, setFilterMap ] = useState<Map<string, string>>(new Map());
    const [ filterList, setFilterList ] = useState([]);
    const [ requestPayload, setRequestPayload ] = useState<InterfaceDiagnosticLogsRequest | null>(null);

    const { t } = useTranslation();

    useEffect(() => {
        const current = getDateTimeWithOffset(timeZone);

        setEndTime(current.toString());
        setStartTime((current - 3600*1000*timeRange).toString());
    }, []);

    useEffect(() => {
        setRequestPayload(
            {
                endTime: endTime,
                filter: searchQuery,
                limit: LOG_FETCH_COUNT,
                startTime: startTime
            }
        );
    }, [ endTime, startTime, timeZone ]);

    useEffect(() => {
        let tempQuery = "";

        if (searchQuery) {
            if (filterQuery) {
                tempQuery = ` and ${searchQuery}`;
            } else {
                tempQuery = searchQuery;
            }
        }
        setRequestPayload({
            endTime: endTime,
            filter: `${filterQuery}${tempQuery}`,
            limit: LOG_FETCH_COUNT,
            startTime: startTime
        });
    }, [ filterQuery ]);

    const scrollRef = useRef<HTMLDivElement>(null);
    const addFilterButtonRef: MutableRefObject<HTMLDivElement> = useRef();
    const { error, list, loading, next, previous } = useFetch(requestPayload);

    /**
     * Handles the behaviour of the infinite scroller
     */
    const handleScroll = (e) => {
        const element = e.target;

        /**
         * When the at the top of the log container
         */
        if (element.scrollTop === 0) {

            if (previous) {
                setRequestPayload({
                    filter: searchQuery,
                    limit: LOG_FETCH_COUNT,
                    previousToken: previous
                });
                setIsPreviousEmpty(false);
                setIsNextEmpty(false);
            } else {
                setIsPreviousEmpty(true);
            }
        }
        /**
         * When the at the bottom of the log container
         */
        if (element.scrollHeight - element.scrollTop === element.clientHeight) {
            if (next) {
                setRequestPayload({
                    filter: searchQuery,
                    limit: LOG_FETCH_COUNT,
                    nextToken: next
                });
                setIsNextEmpty(false);
                setIsPreviousEmpty(false);
            } else {
                setIsNextEmpty(true);
            }
        }
    };

    /**
     * Build filter query
     */
    const buildFilterQuery = () => {
        let tempQuery = "";

        filterMap.forEach((val) => {
            if (tempQuery) {
                tempQuery = `${tempQuery} and logentry co ${val}`;
            } else {
                tempQuery = `logentry co ${val}`;
            }
        });
        setFilterQuery(tempQuery);
    };

    /**
     * Search logs
     *
     * @param query - search query with filters
     */
    const handleSearch = (query: string) => {
        setSearchQuery(query);
        if (filterQuery) {
            query = ` and ${query}`;
        }
        setRequestPayload({
            endTime: endTime,
            filter: `${filterQuery}${query}`,
            limit: LOG_FETCH_COUNT,
            startTime: startTime
        });
    };

    /**
     * Handles the `onSearchQueryClear` callback action.
     */
    const handleSearchQueryClear = (): void => {
        setTriggerClearQuery(true);
        setSearchQuery("");
        setRequestPayload({
            endTime: endTime,
            filter: filterQuery,
            limit: LOG_FETCH_COUNT,
            startTime: startTime
        });
    };

    /**
     * Handles the search query submit.
     *
     * @param processQuery - (boolean) Flag to enable query processing.
     * @param query - (string) Search query.
     */
    const handleSearchQuerySubmit = (processQuery: boolean, query: string): void => {
        if (!processQuery) {
            handleSearch(query);

            return;
        }

        handleSearch(SearchUtils.buildSearchQuery(query));
    };

    /**
     * Handles searching logs with multiple filters
     */
    const handleFilter = (query: InterfaceLogsFilter): void => {
        const tempMap = filterMap;

        tempMap.set(query.property, query.value);
        setFilterMap(tempMap);
        buildFilterQuery();

        const arr: { key: string, value:string }[] = [];

        tempMap.forEach((value, key) => {
            arr.push({ key, value });
        });

        setFilterList(arr);
        setIsAddFilterFormSubmitted(true);
    };

    /**
     * Remove specific filter attributes on click delete button
     * in filter pills
     */
    const removeFilter = (filterKey: string): void => {
        const tempMap = filterMap;

        tempMap.delete(filterKey);
        setFilterMap(tempMap);
        buildFilterQuery();

        const arr: { key: string, value:string }[] = [];

        tempMap.forEach((value, key) => {
            arr.push({ key, value });
        });

        setFilterList(arr);
    };

    /**
     * Clear all filter attributes
     */

    const clearAllFilters = (): void => {
        const tempMap = filterMap;

        tempMap.clear();
        setFilterMap(tempMap);
        buildFilterQuery();
        setFilterList([]);
    };

    /**
     * Returns search component
     */
    const advancedSearchFilter = (): ReactElement => (
        <AdvancedSearch
            aligned="left"
            fill="white"
            onSearchQuerySubmit={ handleSearchQuerySubmit }
            placeholder={ "Search Logs" }
            defaultSearchStrategy="logentry co %search-value%"
            triggerClearQuery={ triggerClearQuery }
            onExternalSearchQueryClear={ handleSearchQueryClear }
        />
    );

    /**
     * Handles the time range according to the dropdown
     */
    const handleTimeRange = (e, { value }) => {
        /**
         * number of milliseconds elapsed since January 1, 1970 00:00:00 UTC
         */
        const current = getDateTimeWithOffset(timeZone);

        setTimeRange(value);
        setEndTime(current.toString());
        /**
         * set start time by subtracting the time range selected
         */
        setStartTime((current - 3600*1000*value).toString());
    };

    const resolveContent = (): ReactElement => {
        if (error) {
            return (
                // TODO: add i18n strings
                <EmptyPlaceholder
                    subtitle={ [ "Couldn't fetch diagnostic logs",
                        "Please try again" ] }
                    title="Something went wrong"
                    image={ getEmptyPlaceholderIllustrations().genericError }
                    imageSize="tiny"
                />
            );
        } else if (searchQuery && list.length === 0) {
            return (
                <EmptyPlaceholder
                    action={ (
                        <LinkButton onClick={ handleSearchQueryClear }>
                            { t("console:develop.placeholders.emptySearchResult.action") }
                        </LinkButton>
                    ) }
                    image={ getEmptyPlaceholderIllustrations().emptySearch }
                    imageSize="tiny"
                    title={ t("console:develop.placeholders.emptySearchResult.title") }
                    subtitle={ [
                        t("console:develop.placeholders.emptySearchResult.subtitles.0", { query: searchQuery }),
                        t("console:develop.placeholders.emptySearchResult.subtitles.1")
                    ] }
                    data-testid={ `${ componentId }-empty-search-placeholder-icon` }
                />
            );
        } else if (filterQuery && list.length === 0) {
            return (
                <EmptyPlaceholder
                    action={ (
                        <LinkButton onClick={ clearAllFilters }>
                            Clear all filters
                        </LinkButton>
                    ) }
                    image={ getEmptyPlaceholderIllustrations().emptySearch }
                    imageSize="tiny"
                    title={ t("console:develop.placeholders.emptySearchResult.title") }
                    subtitle={ [
                        t("console:develop.placeholders.emptySearchResult.subtitles.0", { query: filterQuery }),
                        t("console:develop.placeholders.emptySearchResult.subtitles.1")
                    ] }
                    data-testid={ `${ componentId }-empty-search-placeholder-icon` }
                />
            );
        } else if (list.length === 0 && (!next || !previous) && !loading) {
            return (
                <EmptyPlaceholder
                    image={ getEmptyPlaceholderIllustrations().emptySearch }
                    imageSize="tiny"
                    title="No logs available"
                    subtitle={ [
                        "We couldn't find any logs in " +
                        `${ TIMERANGE_DROPDOWN_OPTIONS.find(el => el.value === timeRange)?.text.toLowerCase() }`
                        ,
                        "Please try a different time range."
                    ] }
                    data-testid={ `${ componentId }-empty-search-placeholder-icon` }
                />
            );
        } else {
            return (
                (<InfiniteScrollContainer
                    handleScroll={ handleScroll }
                    scrollRef={ scrollRef }
                    logs={ list }
                    loading={ loading }
                    rowHeight= { LOG_ROW_HEIGHT }
                    logCount={ LOG_FETCH_COUNT }
                    isPreviousEmpty = { isPreviousEmpty }
                    isNextEmpty = { isNextEmpty }
                    setSearchQuery = { handleFilter }
                />)
            );
        }
    };

    return (
        <div className="diagnostic-logs">
            <PageLayout
                title={ "Diagnostic Logs" }
                pageTitle="Monitor"
                description={
                    "Logs for developers to self-diagnose organization issues"
                }
                data-componentid={ `${componentId}-layout` }
            >
                <ListLayout
                    // eslint-disable-next-line @typescript-eslint/no-empty-function
                    onPageChange={ () => {} }
                    advancedSearch={ advancedSearchFilter() }
                    rightActionPanel={
                        (
                            <div>
                                <Dropdown
                                    compact
                                    selection
                                    options={ TIMERANGE_DROPDOWN_OPTIONS }
                                    onChange={ handleTimeRange }
                                    defaultValue={ 0.5 }
                                />
                            </div>
                        ) }
                    totalPages={ 1 }
                    showPagination={ false }
                    data-testid={ `${componentId}-list-layout` }
                >
                    <>
                        <div className="top-toolbar">
                            <div className="buttons-row" ref={ addFilterButtonRef }>
                                <AdvancedFilterPopup
                                    addFilterButtonRef={ addFilterButtonRef }
                                    setSearchQuery={ handleFilter }
                                    isFormSubmitted={ isAddFilterFormSubmitted }
                                    resetSubmitState={ () => setIsAddFilterFormSubmitted(false) }
                                />
                                <LinkButton
                                    basic
                                    compact
                                    primary={ false }
                                    data-componentid={ `${ componentId }-clear-filter-button` }
                                    data-testid={ `${ componentId }-clear-filter-button` }
                                    onClick={ clearAllFilters }
                                    disabled={ !filterQuery }
                                >
                                    Clear all filters
                                </LinkButton>
                            </div>
                            <Label.Group>
                                { filterList && filterList.map((value, key) =>
                                    (<Label key={ key } className="filter-pill">
                                        { value.key }
                                        <Label.Detail>{ value.value }</Label.Detail>
                                        <Icon name="delete" onClick={ () => removeFilter(value.key) }></Icon>
                                    </Label>)
                                ) }
                            </Label.Group>
                        </div>
                        { resolveContent() }
                    </>
                </ListLayout>
            </PageLayout>
        </div>
    );
};

/**
 * Default props for the component.
 */
LogsPage.defaultProps = {
    "data-componentid": "logs-page"
};

export default LogsPage;
